import { defineStore } from 'pinia'
import { ref, computed } from 'vue'
import { gameService } from '@/services/game'
import { Game } from '@/types/game'

export const useGameStore = defineStore('game', () => {
  // 状态
  const games = ref<Game[]>([]) // 游戏列表
  const currentGame = ref<Game | null>(null) // 当前选中的游戏
  const loading = ref(false) // 加载状态
  const error = ref<string | null>(null) // 错误信息

  // 计算属性
  const hotGames = computed(() => games.value.filter((game) => game.isHot)) // 热门游戏
  const newGames = computed(() => games.value.filter((game) => game.isNew)) // 新游戏

  // 获取游戏列表
  async function fetchGames() {
    loading.value = true
    error.value = null

    try {
      const response = await gameService.getGames()

      if (response.success && response.data) {
        games.value = response.data
      } else {
        error.value = response.message || '获取游戏列表失败'
        games.value = []
      }
    } catch (e) {
      console.error('获取游戏列表失败:', e)
      error.value = '获取游戏列表失败'
      games.value = []
    } finally {
      loading.value = false
    }
  }

  // 获取游戏详情
  async function getGameDetail(id: string): Promise<Game | null> {
    // 如果当前正在加载，直接返回 null 防止重复请求
    if (loading.value) {
      console.warn('详情加载中，避免重复请求:', id)
      return null
    }

    // 优先从缓存中查找
    const cachedGame = games.value.find((game) => game.id === id)
    if (cachedGame) {
      console.log('从缓存中找到游戏:', cachedGame)

      // 如果缓存中的游戏缺少详细信息，从 API 补充数据
      if (!cachedGame.description || !cachedGame.title) {
        console.log('缓存数据不完整，从 API 获取详情:', id)
        return await fetchGameDetailFromApi(id, cachedGame)
      }

      // 如果缓存数据完整，直接返回
      currentGame.value = cachedGame
      return cachedGame
    }

    // 缓存中没有，直接从 API 获取
    return await fetchGameDetailFromApi(id)
  }

  // 从 API 获取游戏详情并更新缓存
  async function fetchGameDetailFromApi(id: string, cachedGame?: Game): Promise<Game | null> {
    loading.value = true
    error.value = null

    try {
      const response = await gameService.getGameDetail(id)

      if (response.success && response.data) {
        const detailGame = response.data

        // 更新缓存中的数据
        if (cachedGame) {
          Object.assign(cachedGame, detailGame) // 合并新数据到缓存
        } else {
          games.value.push(detailGame) // 如果缓存中没有，直接添加到列表
        }

        currentGame.value = detailGame
        return detailGame
      } else {
        error.value = response.message || '获取游戏详情失败'
        return null
      }
    } catch (e) {
      console.error('获取游戏详情失败:', e)
      error.value = '获取游戏详情失败'
      return null
    } finally {
      loading.value = false
    }
  }

  // 设置当前游戏
  function setCurrentGame(game: Game | null) {
    currentGame.value = game
  }

  // 清除游戏数据
  function clearGames() {
    games.value = []
    currentGame.value = null
    error.value = null
  }

  return {
    // 状态
    games,
    currentGame,
    loading,
    error,

    // 计算属性
    hotGames,
    newGames,

    // 方法
    fetchGames,
    getGameDetail,
    setCurrentGame,
    clearGames
  }
})