// stores/user.ts
import { defineStore } from 'pinia'
import Taro from '@tarojs/taro'
import { userService } from '@/services/user'
import { type UserInfo, UserRole } from '@/types/user'
import { ref, computed } from 'vue'

export const useUserStore = defineStore('user', () => {
  // 基本用户信息
  const openid = ref('')
  const nickname = ref('')
  const avatar = ref('')
  const role = ref<UserRole>(UserRole.USER)
  const isRegistered = ref(false)
  
  // 当前状态
  const currentRoom = ref<string | null>(null)
  const currentGameId = ref<string | null>(null)
  const loading = ref(false)
  const error = ref<string | null>(null)
  
  // 经验值系统
  const points = ref(0)
  const exp = ref(0)
  const level = ref(1)
  const nextLevelExp = ref(100)
  
  // 游戏统计数据
  const gameCount = ref(0)
  const roomCount = ref(0)

  // 计算属性
  const userInfo = computed<UserInfo>(() => ({
    openid: openid.value,
    nickname: nickname.value,
    avatar: avatar.value,
    role: role.value
  }))
  
  const expPercentage = computed(() => {
    return (exp.value / nextLevelExp.value) * 100
  })

  // 保存/加载用户数据
  function saveUserToStorage() {
    const userData = {
      openid: openid.value,
      nickname: nickname.value,
      avatar: avatar.value,
      role: role.value,
      isRegistered: isRegistered.value,
      currentRoom: currentRoom.value,
      currentGameId: currentGameId.value,
      points: points.value,
      exp: exp.value,
      level: level.value,
      nextLevelExp: nextLevelExp.value,
      gameCount: gameCount.value,
      roomCount: roomCount.value
    }
    
    Taro.setStorageSync('userInfo', JSON.stringify(userData))
    console.log('userData', userData)
    return userData
  }
  
  function loadUserFromStorage(): boolean {
    try {
      const userInfo = Taro.getStorageSync('userInfo')
      if (!userInfo) return false
      
      const data = JSON.parse(userInfo)
      openid.value = data.openid || ''
      nickname.value = data.nickname || ''
      avatar.value = data.avatar || ''
      role.value = data.role || UserRole.USER
      isRegistered.value = !!data.isRegistered
      currentRoom.value = data.currentRoom || null
      currentGameId.value = data.currentGameId || null
      
      points.value = data.points || 0
      exp.value = data.exp || 0
      level.value = data.level || 1
      nextLevelExp.value = data.nextLevelExp || 100
      
      gameCount.value = data.gameCount || 0
      roomCount.value = data.roomCount || 0
      
      console.log('loadUserFromStorage', userInfo)
      return true
    } catch (error) {
      console.error('加载用户信息失败:', error)
      return false
    }
  }
  
  // 设置用户数据 - 用于从API设置完整用户数据
  function setUserData(userData: any) {
    if (!userData) return
    
    openid.value = userData.openid || openid.value
    nickname.value = userData.nickname || nickname.value
    avatar.value = userData.avatar || avatar.value
    role.value = userData.role || role.value
    isRegistered.value = userData.isRegistered !== undefined ? userData.isRegistered : isRegistered.value
    
    // 设置可选的扩展数据
    if (userData.points !== undefined) points.value = userData.points
    if (userData.exp !== undefined) exp.value = userData.exp
    if (userData.level !== undefined) level.value = userData.level
    if (userData.gameCount !== undefined) gameCount.value = userData.gameCount
    if (userData.roomCount !== undefined) roomCount.value = userData.roomCount
    
    saveUserToStorage()
  }
  
  // 初始化用户
  async function initUser() {
    loading.value = true
    error.value = null

    try {
      // 1. 尝试从本地存储加载
      const localDataLoaded = loadUserFromStorage()
      
      // 2. 如果已有数据且注册，直接返回
      if (localDataLoaded && isRegistered.value) {
        return true
      }
      
      // 3. 如果有openid但未注册，保持现状
      if (localDataLoaded && openid.value) {
        return true
      }
      
      // 4. 否则，执行静默登录获取openid
      return await silentLogin()
    } catch (e) {
      console.error('初始化用户失败:', e)
      error.value = '初始化用户失败'
      return false
    } finally {
      loading.value = false
    }
  }
  
  // 静默登录 - 仅获取openid
  async function silentLogin() {
    try {
      // userService处理mock或真实环境逻辑
      const result = await userService.login()
      
      if (result.success && result.data) {
        // 设置用户数据
        setUserData(result.data)
        return true
      } else {
        error.value = result.message || '登录失败'
        return false
      }
    } catch (e) {
      console.error('静默登录失败:', e)
      error.value = '静默登录失败'
      return false
    }
  }
  
  // 注册用户 - 获取用户信息
  async function registerUser() {
    loading.value = true
    error.value = null
    
    try {
      // 确保有openid
      if (!openid.value) {
        const loginSuccess = await silentLogin()
        if (!loginSuccess) return false
      }
      
      // 调用userService处理注册逻辑（内部处理mock/真实环境）
      const result = await userService.register(openid.value)
      
      if (result.success && result.data) {
        // 设置用户数据
        setUserData(result.data)
        return true
      } else {
        error.value = result.message || '注册失败'
        return false
      }
    } catch (e) {
      console.error('用户注册失败:', e)
      error.value = '用户注册失败'
      return false
    } finally {
      loading.value = false
    }
  }
  
  // 经验值系统
  function addPoints(amount: number) {
    points.value += amount
    saveUserToStorage()
  }
  
  function addExp(amount: number) {
    exp.value += amount
    checkLevelUp()
    saveUserToStorage()
    return { level: level.value, exp: exp.value, nextLevelExp: nextLevelExp.value }
  }
  
  function checkLevelUp() {
    while (exp.value >= nextLevelExp.value) {
      level.value++
      exp.value -= nextLevelExp.value
      nextLevelExp.value = Math.floor(nextLevelExp.value * 1.2)
    }
  }
  
  // 统计数据
  function recordGameParticipation() {
    gameCount.value++
    saveUserToStorage()
  }
  
  function recordRoomCreation() {
    roomCount.value++
    saveUserToStorage()
  }
  
  // 简单setter方法
  function setCurrentRoom(roomId: string) {
    currentRoom.value = roomId
    saveUserToStorage()
  }
  
  function setCurrentGame(gameId: string | null) {
    currentGameId.value = gameId
    saveUserToStorage()
  }
  
  function setRole(newRole: UserRole) {
    role.value = newRole
    saveUserToStorage()
  }
  
  // 重置/清除
  function logout() {
    openid.value = ''
    nickname.value = ''
    avatar.value = ''
    isRegistered.value = false
    currentRoom.value = null
    currentGameId.value = null
    points.value = 0
    exp.value = 0
    level.value = 1
    nextLevelExp.value = 100
    gameCount.value = 0
    roomCount.value = 0
    
    Taro.removeStorageSync('userInfo')
  }
  
  function resetToNormalUser() {
    role.value = UserRole.USER
    saveUserToStorage()
  }
  
  function clearGameSession() {
    currentRoom.value = null
    currentGameId.value = null
    saveUserToStorage()
  }

  return {
    // 状态
    openid,
    nickname,
    avatar,
    role,
    isRegistered,
    currentRoom,
    currentGameId,
    loading,
    error,
    userInfo,
    
    // 经验值系统
    points,
    exp,
    level,
    nextLevelExp,
    expPercentage,
    
    // 游戏统计
    gameCount,
    roomCount,
    
    // 方法
    initUser,
    registerUser,
    silentLogin,
    setRole,
    setCurrentRoom,
    setCurrentGame,
    logout,
    resetToNormalUser,
    clearGameSession,
    
    // 经验值系统方法
    addPoints,
    addExp,
    
    // 统计方法
    recordGameParticipation,
    recordRoomCreation
  }
})